from GFM313_Tx_tests.General.GFM313_base import TxTestsBase
from GFM313_Tx_tests.General.GFM313_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException
import csv


class TC62(TxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset_analyzer()
        self.frequency = 1000
        """Set center frequency in MHz"""
        self.testmodel = 'NR-FR1-TM1_1__FDD_10MHz_15kHz'
        """Set test model string"""
        self.reference_level_offset = 0
        """Set reference level offset to compensate external attenuation (in dB)"""

    def start_measurement(self):
        """
        This function starts the measurement.
         :return:
        """
        try:
            self.analyzer.write_str_with_opc('INST:SEL NR5G')
            self.analyzer.write_str_with_opc('CONF:MEAS EVM')
            self.analyzer.write_float(':SENS:FREQ:CENT', self.frequency * 1000000)
            self.analyzer.write_float('INPut:ATTenuation', self.reference_level_offset)
            self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{self.testmodel}'")
            self.analyzer.write_str_with_opc('CONF:DL:IDC 1')
            self.analyzer.write_str(':INIT:CONT OFF')
            self.analyzer.write_str_with_opc('INIT:IMM')
        except RsInstrException as e:
            raise RsException(e.args[0])

    def return_results(self):
        """This function returns the maximum output power"""
        try:
            maximum_power = self.analyzer.query_float('FETC:SUMM:POW:MAX?')
        except RsInstrException as e:
            raise RsException(e.args[0])
        return maximum_power

    def csv_export(self):
        """
        Exports the results summary to csv file (project directory).
        :return:
        """
        with open('results_62_BSPower.csv', 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            results = [self.return_results()]
            header = ['BS output power [dBm]']
            entries = [header, results]
            writer.writerows(entries)
