from GFM313_Tx_tests.General.GFM313_base import TxTestsBase
from GFM313_Tx_tests.General.GFM313_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException
import csv


class TC654(TxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset_analyzer()

        self.frequency = 1000
        """Set center frequency in MHz"""
        self.testmodel = 'NR-FR1-TM1_1__FDD_10MHz_15kHz'
        """Set test model string"""
        self.layer = 'LC11'
        """Select layer mapping: 'LC11', 'LC21', 'LC31', 'LC41', etc."""

        self.antenna_port = 'AP1001'
        """Select antenna port that should be measured: 'AP1000', 'AP1001', etc.'"""

    def start_measurement(self):
        """
        This function starts the measurement.
         :return:
        """
        try:
            self.analyzer.write_str_with_opc('INST:SEL NR5G')
            self.analyzer.write_str_with_opc('CONF:MEAS TAER')
            self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{self.testmodel}'")
            self.analyzer.write_float(':SENS:FREQ:CENT', self.frequency * 1000000)
            self.analyzer.write_str(f'CONF:DL:FRAM1:BWP0:SLOT0:ALL0:CLM {self.layer}')
            self.analyzer.write_str(':INIT:CONT OFF')
            self.analyzer.write_str_with_opc('INIT:IMM')

        except RsInstrException as e:
            raise RsException(e.args[0])

    def return_results(self):
        """
        This function returns the time alignment error measurement results in an array structure. [Antenna port,
        Minimum, Mean, Maximum] :return:
        """
        try:
            tae_min = self.analyzer.query_float(f'FETC:TAER:CC:{self.antenna_port}:MIN?')
            tae_max = self.analyzer.query_float(f'FETC:TAER:CC:{self.antenna_port}:MAX?')
            tae_av = self.analyzer.query_float(f'FETC:TAER:CC:{self.antenna_port}:AVER?')
        except RsInstrException as e:
            raise RsException(e.args[0])
        results = [self.antenna_port, tae_min, tae_av, tae_max]
        return results

    def csv_export(self):
        """
        Exports the results to csv file (project directory).
        :return:
        """
        with open('results_654_TAE.csv', 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            results = self.return_results()
            header = ['Antenna port', 'TAE minimum [s]', 'TAE mean [s]', 'TAE maximum [s]']
            entries = [header, results]
            writer.writerows(entries)
