from GFM313_Tx_tests.General.GFM313_base import TxTestsBase
from GFM313_Tx_tests.General.GFM313_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException
import csv


class TC662(TxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset_analyzer()
        self.frequency = 1000
        """Set center frequency in MHz"""
        self.trigger = 'IMM'
        """Select trigger source: Free run ('IMM'), External ('EXT', 'EXT2' or 'EXT3')"""
        self.channel_bw = 10
        """Set channel bandwidth in MHz"""

    def start_measurement(self):
        """
        This function starts the measurement.
         :return:
        """
        try:
            self.analyzer.write_str_with_opc('INST:SEL SAN')
            self.analyzer.write_str_with_opc('CALC:MARK:FUNC:POW:SEL OBAN')
            self.analyzer.write_float(':SENS:FREQ:CENT', self.frequency * 1000000)
            self.analyzer.write_float(':SENS:POW:ACH:BWID', self.channel_bw * 1000000)
            self.analyzer.write_str(f'TRIG:SOUR {self.trigger}')
            self.analyzer.write_str(':INIT:CONT OFF')
            self.analyzer.write_str_with_opc('INIT:IMM')

        except RsInstrException as e:
            raise RsException(e.args[0])

    def return_results(self):
        """
        This function returns the occupied bandwidth in Hz.
        :return:
        """
        try:
            obw = self.analyzer.query_float_with_opc('CALC:MARK:FUNC:POW:RES? OBAN')
        except RsInstrException as e:
            raise RsException(e.args[0])
        results = obw
        return results

    def csv_export(self):
        """
        Exports the results summary to csv file (project directory).
        :return:
        """
        with open('results_662_OBW.csv', 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            results = [self.return_results()]
            header = ['Occupied bandwidth [Hz]']
            entries = [header, results]
            writer.writerows(entries)
