from GFM313_Tx_tests.General.GFM313_base import TxTestsBase
from GFM313_Tx_tests.General.GFM313_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException
import csv


class TC663(TxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset_analyzer()
        self.frequency = 1000
        """Set center frequency in MHz"""
        self.testmodel = 'NR-FR1-TM1_1__FDD_10MHz_15kHz'
        """Set test model string"""
        self.trigger = 'IMM'
        """Select trigger source: Free run ('IMM'), External ('EXT', 'EXT2' or 'EXT3')"""
        self.channel_bw = 10
        """Set channel bandwidth in MHz"""

    def start_measurement(self):
        """
        This function starts the measurement.
         :return:
        """
        try:
            self.analyzer.write_str_with_opc('INST:SEL NR5G')
            self.analyzer.write_str_with_opc('CONF:MEAS ACLR')
            self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{self.testmodel}'")
            self.analyzer.write_float(':SENS:FREQ:CENT', self.frequency * 1000000)
            self.analyzer.write_str(f'TRIG:SOUR {self.trigger}')
            self.analyzer.write_str(':SENS:POW:NCOR ON')
            self.analyzer.write_str(':INIT:CONT OFF')
            self.analyzer.write_str_with_opc('INIT:IMM')

        except RsInstrException as e:
            raise RsException(e.args[0])

    def return_results(self):
        """
        This function returns the results in an array structure. [Channel power (dBm), Power of lower adjacent channel
        (dBc), Power of upper adjacent channel (dBc), Power of lower alternate channel 1 (dBc),
        Power of upper alternate channel 1 (dBc)]
        :return:
        """
        try:
            power = self.analyzer.query_bin_or_ascii_float_list_with_opc('CALC:MARK:FUNC:POW:RES? ACP')
        except RsInstrException as e:
            raise RsException(e.args[0])
        results = power
        return results

    def csv_export(self):
        """
        Exports the results to csv file (project directory).
        :return:
        """
        with open('results_663_ACLR.csv', 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            results = self.return_results()
            header = ['Channel power [dBm]', 'Power of lower adjacent channel [dBc]',
                      'Power of upper adjacent channel [dBc]', 'Power of lower alternate channel 1 [dBc]',
                      'Power of upper alternate channel 1 [dBc]']
            entries = [header, results]
            writer.writerows(entries)
