from GFM313_Tx_tests.General.GFM313_base import TxTestsBase
from GFM313_Tx_tests.General.GFM313_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException
import csv


class TC665(TxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset_analyzer()
        self.frequency_5thharmonic = None
        """Upper frequency edge of the 5th harmonic [GHz]"""
        self.category = 'A'
        """Select category: 'A' or 'B'"""

    def start_measurement(self):
        """
        This function starts the measurement.
         :return:
        """
        try:
            self.analyzer.write_str_with_opc('INST:SEL SAN')
            self.analyzer.write_str_with_opc('INIT:SPUR')
            self.analyzer.write_str(':SWE:MODE LIST')
            self.analyzer.write_str(':INIT:CONT OFF')

            if self.category == 'A':
                self.analyzer.write_str(':SENS:LIST:RANG1:FREQ:STAR 9KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG1:FREQ:STOP 150KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG1:BAND 1KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG1:LIM:STAR -13')
                self.analyzer.write_str(':SENS:LIST:RANG1:LIM:STOP -13')

                self.analyzer.write_str(':SENS:LIST:RANG2:FREQ:STAR 150KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG2:FREQ:STOP 30MHZ')
                self.analyzer.write_str(':SENS:LIST:RANG2:BAND 10KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG2:LIM:STAR -13')
                self.analyzer.write_str(':SENS:LIST:RANG2:LIM:STOP -13')

                self.analyzer.write_str(':SENS:LIST:RANG3:FREQ:STAR 30MHZ')
                self.analyzer.write_str(':SENS:LIST:RANG3:FREQ:STOP 1GHZ')
                self.analyzer.write_str(':SENS:LIST:RANG3:BAND 100KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG3:LIM:STAR -13')
                self.analyzer.write_str(':SENS:LIST:RANG3:LIM:STOP -13')

                self.analyzer.write_str(':SENS:LIST:RANG4:FREQ:STAR 1GHZ')
                self.analyzer.write_str(':SENS:LIST:RANG4:FREQ:STOP 12.75GHZ')
                self.analyzer.write_str(':SENS:LIST:RANG4:BAND 1MHZ')
                self.analyzer.write_str(':SENS:LIST:RANG4:LIM:STAR -13')
                self.analyzer.write_str(':SENS:LIST:RANG4:LIM:STOP -13')

                self.analyzer.write_str(':SENS:LIST:RANG5:FREQ:STAR 12.75GHz')
                if self.frequency_5thharmonic is not None:
                    self.analyzer.write_float(':SENS:LIST:RANG5:FREQ:STOP ', self.frequency_5thharmonic * 1000000000)
                self.analyzer.write_str(':SENS:LIST:RANG5:BAND 1MHZ')
                self.analyzer.write_str(':SENS:LIST:RANG5:LIM:STAR -13')
                self.analyzer.write_str(':SENS:LIST:RANG5:LIM:STOP -13')

            elif self.category == 'B':
                self.analyzer.write_str(':SENS:LIST:RANG1:FREQ:STAR 9KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG1:FREQ:STOP 150KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG1:BAND 1KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG1:LIM:STAR -36')
                self.analyzer.write_str(':SENS:LIST:RANG1:LIM:STOP -36')

                self.analyzer.write_str(':SENS:LIST:RANG2:FREQ:STAR 150KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG2:FREQ:STOP 30MHZ')
                self.analyzer.write_str(':SENS:LIST:RANG2:BAND 10KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG2:LIM:STAR -36')
                self.analyzer.write_str(':SENS:LIST:RANG2:LIM:STOP -36')

                self.analyzer.write_str(':SENS:LIST:RANG3:FREQ:STAR 30MHZ')
                self.analyzer.write_str(':SENS:LIST:RANG3:FREQ:STOP 1GHZ')
                self.analyzer.write_str(':SENS:LIST:RANG3:BAND 100KHZ')
                self.analyzer.write_str(':SENS:LIST:RANG3:LIM:STAR -36')
                self.analyzer.write_str(':SENS:LIST:RANG3:LIM:STOP -36')

                self.analyzer.write_str(':SENS:LIST:RANG4:FREQ:STAR 1GHZ')
                self.analyzer.write_str(':SENS:LIST:RANG4:FREQ:STOP 12.75GHZ')
                self.analyzer.write_str(':SENS:LIST:RANG4:BAND 1MHZ')
                self.analyzer.write_str(':SENS:LIST:RANG4:LIM:STAR -30')
                self.analyzer.write_str(':SENS:LIST:RANG4:LIM:STOP -30')

                self.analyzer.write_str(':SENS:LIST:RANG5:FREQ:STAR 12.75GHz')
                if self.frequency_5thharmonic is not None:
                    self.analyzer.write_float(':SENS:LIST:RANG5:FREQ:STOP ', self.frequency_5thharmonic * 1000000000)
                self.analyzer.write_str(':SENS:LIST:RANG5:BAND 1MHZ')
                self.analyzer.write_str(':SENS:LIST:RANG5:LIM:STAR -30')
                self.analyzer.write_str(':SENS:LIST:RANG5:LIM:STOP -30')

            self.analyzer.write_str(':SENS:LIST:XADJ;*WAI')
            self.analyzer.write_str_with_opc('INIT:SPUR')

        except RsInstrException as e:
            raise RsException(e.args[0])

    def return_results(self):
        """
        This function returns the results in an array structure. ['Range Low (Hz)', 'Range Up (Hz)', 'RBW (Hz)',
         'PeakFreq (Hz)', 'PowerAbs (dBm)', 'PowerRel (dB)', 'PowerDelta (dB)', 'LimitCheck (1/0)']
        :return:
        """
        try:
            results = self.analyzer.query_bin_or_ascii_float_list_with_opc('TRAC:DATA? LIST')
        except RsInstrException as e:
            raise RsException(e.args[0])
        return [results[1:9], results[276:284], results[551:559], results[826:834], results[1101:1109]]

    def csv_export(self):
        """
        Exports the results to csv file (project directory).
        :return:
        """
        with open('results_665_SPUR.csv', 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            results = self.return_results()
            header = ['Range Low [Hz]', 'Range Up [Hz]', 'RBW [Hz]', 'PeakFreq [Hz]', 'PowerAbs [dBm]', 'PowerRel [dB]',
                      'PowerDelta [dB]', 'LimitCheck [1/0]']
            entries = [header, results[0], results[1], results[2], results[3], results[4]]
            writer.writerows(entries)
