from GFM313_Tx_tests.General.GFM313_base import TxTestsBase
from GFM313_Tx_tests.General.GFM313_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException


class TC67(TxTestsBase):
    def __init__(self, resource_name_generator: str):
        super().__init__(resource_name_generator=resource_name_generator)
        self.resource_name = resource_name_generator
        super().reset_generator()

        self.bs_class = None
        """Select base station class: Wide ('Wide'), Medium ('MED') or Local ('LOC')"""

        self.trigger_config = None
        """Select trigger configuration: Armed auto ('AAUT') or unchanged ('UNCH')"""
        self.marker_config = None
        """Select marker configuration: Radio frame start ('FRAM') or unchanged ('UNCH')"""

        self.frequency = None
        """Set center frequency in MHz"""
        self.channel_bw = None
        """Set channel bandwidth in MHz"""
        self.power = None
        """Set power level in dBm"""

        self.allocation = None
        """Select position of interfering signal: At higher frequencies ('HIGH') or at lower frequencies ('LOW')"""

        self.interferer_frequencyband = None
        """Select band of interfering signal: 'N1', 'N2', ..., 'N86'"""
        self.interferer_offsetfactor = None
        """Set the offset factor for the interfering signal"""
        self.interferer_duplex = None
        """Select the duplexing mechanism used for the interfering signal: 'FDD' or 'TDD'"""

    def apply_swm_configuration(self):
        """
        This function applies the configuration and sends the remote control commands to the connected signal generator
        (SMW).
        :return:
        """
        try:
            self.generator.write_str(':BB:NR5G:TCW:TC TS381411_TC67')
            if self.bs_class is not None:
                self.generator.write_str(f':BB:NR5G:TCW:BSCL {self.bs_class}')

            super().instrument_settings(self.trigger_config, self.marker_config)

            if self.frequency is not None:
                self.generator.write_float(':BB:NR5G:TCW:WS:RFFR', self.frequency * 1000000)
            if self.channel_bw is not None:
                self.generator.write_str(f':BB:NR5G:TCW:WS:CBW BW{self.channel_bw}')
            if self.power is not None:
                self.generator.write_float(':BB:NR5G:TCW:WS:PLEV', self.power)

            if self.allocation is not None:
                self.generator.write_str(f':BB:NR5G:TCW:FA:FRAL {self.allocation}')

            if self.interferer_frequencyband is not None:
                self.generator.write_str(f':BB:NR5G:TCW:IS:BAND {self.interferer_frequencyband}')
            if self.interferer_offsetfactor is not None:
                self.generator.write_str(f':BB:NR5G:TCW:IS:OFN OFN_{self.interferer_offsetfactor}')
            if self.interferer_duplex is not None:
                self.generator.write_str(f':BB:NR5G:TCW:IS:DUPL {self.interferer_duplex}')

            self.generator.write_str(':BB:NR5G:TCW:APPL')
        except RsInstrException as e:
            raise RsException(e.args[0])
